-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

-- In other words, if you are a modder feel free to reuse this code with above license

--If you reload Lua with CTRL+L use thise in the GE console `extensions.mainLevel.onClientStartMission()

local windowOpen = ui_imgui.BoolPtr(true)

local M = {}
M.dependencies = {"ui_imgui"}

local logTag = 'addon_ck_ckgraphics'
local im = ui_imgui
local ffi = require("ffi")

--independent GUI
local showUI = nil
local tempFloat = nil

--ssao
local ssaoenabled = false
local isssaoenabled = false
--pssm
local pssmenabled = false
local pssmlenabled = false
local ispssmenabled = false
local ispsslmenabled = false
--terdet
local terenabled = false
local isterenabled = false
local terlenabled = false
local isterlenabled = false
--details
local detailenabled = false
local isdetailenabled = false
--textures
local textenabled = false
local istextenabled = false

--settings
local forcedsettings = false
local isforcedsettings = false
local disabledsettings = false
local isdisabledsettings = false
--onload
local LOD
local LODDEF
local pssmquality
local SQ
local lodBias = im.FloatPtr(0)
local grassBias = im.FloatPtr(0)
local grass
local GQ
local terquat
local LT
local detailCount
local DC
local LQ
local tRL
local rTS
local dS

--game version
local ver = nil
local bigver = nil

--platforms
local isVulkan = 0
local isLinux = 0

--settings
local settingsPath = '/settings/ckgraphics.json'

local tool_version = "2.2"
local appTitle = " CK Graphics Settings - ".. tool_version .." - ".. beamng_arch

--game version check
local function gameVER()
  ver = beamng_version
  bigver = string.format("%0.4s", ver)
  --print(bigver)

end

local function settingsSave()
  local s = {
    forcedsettings = forcedsettings,
    ssaoenabled = ssaoenabled,
    pssmenabled = pssmenabled,
    terenabled = terenabled,
    pssmlenabled = pssmlenabled,
    detailenabled = detailenabled,
    terlenabled = terlenabled,
    textenabled = textenabled,
    disabledsettings = disabledsettings,
    grass = grass,
    LOD = LOD,
  }
  jsonWriteFile(settingsPath, s, true)

 end

local function settingsLoad()
  local s = jsonReadFile(settingsPath)
  if s then
    disabledsettings = s.disabledsettings or false
    if disabledsettings == true then
      log('W', '', 'User does not want to load settings' )
    else
    log('I', '', 'Applying settings' )
    forcedsettings = s.forcedsettings or false
    ssaoenabled = s.ssaoenabled or false
    pssmenabled = s.pssmenabled or false
    terenabled = s.terenabled or false
    pssmlenabled = s.pssmlenabled or false
    detailenabled = s.detailenabled or false
    terlenabled = s.terlenabled or false
    textenabled = s.textenabled or false
    grass = s.grass or grass or 1
    LOD = s.LOD or LODDEF or 1
    TorqueScriptLua.setVar("$pref::GroundCover::densityScale", grass)
    TorqueScriptLua.setVar("$pref::TS::detailAdjust", LOD)
    end
  end
end

--get api info to prevent vk
local function getGFX()
  gameVER()
  --print(gfx)
  if bigver == "0.22" or bigver == "0.21" or bigver == "0.20" then
    isVulkan = 2
    log('E', '', 'Cannot retrieve API information' )
    log('E', '', 'Old game version detected' )
  else
    local gfx = Engine.Render.getAdapterType()
    if not string.match(gfx, "Direct3D11") then
      isVulkan = 1
      log('W', '', 'Vulkan API mode detected' )

      else
      log('I', '', 'Vulkan API mode not detected' )
    end
  end
end

--get os version
local function getOS()
  local os = Engine.Platform.getOSInfo()
  --print(dump(os))
  --print(os.shortname)
  if os.type == "linux" then
    isLinux = 1
    log('I', '', 'Linux Detected' )
  elseif os.type == "windows" then
    log('I', '', 'Windows Detected' )
  else
    log('W', '', 'Couldnt get os type' )
    isLinux = 2
  end
end

local function onExtensionLoaded()
  getGFX(gfx)
  getOS(os)
  if isVulkan ~= 2 then
    log('I', '', 'Graphics data is loaded' )
    LOD = tonumber(TorqueScriptLua.getVar("$pref::TS::detailAdjust"))
    pssmquality = tonumber(TorqueScriptLua.getVar("$pref::Shadows::textureScalar"))
    grass = tonumber(TorqueScriptLua.getVar("$pref::GroundCover::densityScale"))
    terquat = tonumber(TorqueScriptLua.getVar("$pref::Terrain::detailScale"))
    detailCount = tonumber(TorqueScriptLua.getVar("$pref::TS::maxDecalCount"))
    tRL = tonumber(TorqueScriptLua.getVar("$pref::Video::textureReductionLevel"))
    rTS = tonumber(TorqueScriptLua.getVar("$pref::Reflect::refractTexScale"))
    dS = tonumber(TorqueScriptLua.getVar("pref::Terrain::detailScale"))
    LODDEF = LOD
    SQ = pssmquality
    GQ = grass
    LT = terquat
    DC = detailCount
    LQ = tRL
    if bigver == "0.28" then
    log('I', '', 'Loading user settings' )
      settingsLoad()
    else
      log('W', '', 'Bad game version, not loading user settings' )
    end
  end
  if showUI == nil then
    showUI = ui_imgui.BoolPtr(false)
  end
end

local function getConsoleNumber(varName)
  local result = getConsoleVariable(varName)
  return result == "" and -1 or result
end

local function onUpdate()

    --background service
    if ssaoenabled == true and isssaoenabled == false then
      isssaoenabled = true
       log('I', '', "high quality gtao")
       settingsSave()
      scenetree.SSAOPostFx:setSamples(32)
    end

    if ssaoenabled == false and isssaoenabled == true then
      isssaoenabled = false
       log('I', '', "low quality gtao")
       settingsSave()
      scenetree.SSAOPostFx:setSamples(16)
    end

    if pssmenabled == true and ispssmenabled == false then
      ispssmenabled = true
       log('I', '', "high quality pssm")
       settingsSave()
       pssmlenabled = false
      TorqueScriptLua.setVar("$pref::Shadows::textureScalar", 4)
    end

    if pssmenabled == false and ispssmenabled == true then
      ispssmenabled = false
       log('I', '', "low quality pssm")
       settingsSave()
      if SQ == nil then
        TorqueScriptLua.setVar("$pref::Shadows::textureScalar", 2)
      else
      TorqueScriptLua.setVar("$pref::Shadows::textureScalar", SQ)
      end
    end

    if terenabled == true and isterenabled == false then
      isterenabled = true
       log('I', '', "high quality ter")
       settingsSave()
       terlenabled = false
      TorqueScriptLua.setVar("$pref::Terrain::lodScale", 0.4)
    end

    if terenabled == false and isterenabled == true then
      isterenabled = false
       log('I', '', "low quality ter")
       settingsSave()
      if LT == nil then
        TorqueScriptLua.setVar("$pref::Terrain::lodScale", 0.75)
      else
        TorqueScriptLua.setVar("$pref::Terrain::lodScale", LT)
      end
    end

    if pssmlenabled == true and ispsslmenabled == false then
      ispsslmenabled = true
       log('I', '', "ultra low quality pssm")
       settingsSave()
       pssmenabled = false
      TorqueScriptLua.setVar("$pref::Shadows::textureScalar", 0.1)
      TorqueScriptLua.setVar("$pref::Shadows::filterMode", "None")
    end

    if pssmlenabled == false and ispsslmenabled == true then
      ispsslmenabled = false
       log('I', '', "low quality pssm")
       settingsSave()
      if SQ == nil then
        TorqueScriptLua.setVar("$pref::Shadows::textureScalar", 1)
        TorqueScriptLua.setVar("$pref::Shadows::filterMode", "SoftShadowHighQuality")
      else
      TorqueScriptLua.setVar("$pref::Shadows::textureScalar", SQ)
      TorqueScriptLua.setVar("$pref::Shadows::filterMode", "SoftShadowHighQuality")
      end
    end

    if detailenabled == true and isdetailenabled == false then
      isdetailenabled = true
       log('I', '', "skip rendering details")
       settingsSave()
      TorqueScriptLua.setVar("$pref::TS::maxDecalCount", 0)
      TorqueScriptLua.setVar("$pref::TS::skipRenderDLs", 8)
    end

    if detailenabled == false and isdetailenabled == true then
      isdetailenabled= false
       log('I', '', "render details")
       settingsSave()
      if DC == nil then
        TorqueScriptLua.setVar("$pref::TS::maxDecalCount", 4000)
        TorqueScriptLua.setVar("$pref::TS::skipRenderDLs", 0)
      else
      TorqueScriptLua.setVar("$pref::TS::maxDecalCount", DC)
      TorqueScriptLua.setVar("$pref::TS::skipRenderDLs", 0)
      end
    end

    if terlenabled == true and isterlenabled == false then
      isterlenabled = true
       log('I', '', "ultra low quality ter")
       settingsSave()
       terenabled = false
      TorqueScriptLua.setVar("$pref::Terrain::lodScale", 8)
    end

    if terlenabled == false and isterlenabled == true then
      isterlenabled = false
       log('I', '', "low quality ter")
       settingsSave()
      if LT == nil then
        TorqueScriptLua.setVar("$pref::Terrain::lodScale", 0.75)
      else
        TorqueScriptLua.setVar("$pref::Terrain::lodScale", LT)
      end
    end

    if textenabled == true and istextenabled == false then
      istextenabled = true
       log('I', '', "ultra low quality textures")
       settingsSave()
      TorqueScriptLua.setVar("$pref::Video::disablePixSpecular", 1)
      TorqueScriptLua.setVar("$pref::Video::textureReductionLevel", 3)
      TorqueScriptLua.setVar("$pref::Reflect::refractTexScale", 0.1)
      TorqueScriptLua.setVar("$pref::Terrain::detailScale", 0.05)
      TorqueScriptLua.setVar("$pref::Video::disablePixSpecular", 0)
      reloadTextures()
    end

    if textenabled == false and istextenabled == true then
      istextenabled = false
       log('I', '', "low quality textures")
       settingsSave()
      if LQ == nil then
        TorqueScriptLua.setVar("$pref::Video::disablePixSpecular", 1)
        TorqueScriptLua.setVar("$pref::Video::textureReductionLevel", LQ)
        TorqueScriptLua.setVar("$pref::Reflect::refractTexScale", rTS)
        TorqueScriptLua.setVar("$pref::Terrain::detailScale", dS)
        TorqueScriptLua.setVar("$pref::Video::disablePixSpecular", 0)
        reloadTextures()
      else
        TorqueScriptLua.setVar("$pref::Video::disablePixSpecular", 1)
        TorqueScriptLua.setVar("$pref::Video::textureReductionLevel", 0)
        TorqueScriptLua.setVar("$pref::Reflect::refractTexScale", 1)
        TorqueScriptLua.setVar("$pref::Terrain::detailScale", 1)
        TorqueScriptLua.setVar("$pref::Video::disablePixSpecular", 0)
        reloadTextures()
      end
    end

    if forcedsettings == true and isforcedsettings == false then
      log('I', '', "Settigns are forced")
      settingsSave()
    end

    if forcedsettings == true then
      isforcedsettings = true

       local LODnow = tonumber(TorqueScriptLua.getVar("$pref::TS::detailAdjust"))
       if LOD then
        if LODnow ~= LOD then
          TorqueScriptLua.setVar("$pref::TS::detailAdjust", LOD)
         end
       end

       local GCnow = tonumber(TorqueScriptLua.getVar("$pref::GroundCover::densityScale"))
       if grass then
        if GCnow ~= grass then
          TorqueScriptLua.setVar("$pref::GroundCover::densityScale", grass)
         end
       end

       if ssaoenabled == true then
        scenetree.SSAOPostFx:setSamples(32)
       end

    end

    if forcedsettings == false and isforcedsettings == true then
      isforcedsettings = false
       log('I', '', "Disabled forcing settings")
       settingsSave()
       local LOD = tonumber(TorqueScriptLua.getVar("$pref::TS::detailAdjust"))
       local GC = tonumber(TorqueScriptLua.getVar("$pref::GroundCover::densityScale"))
    end

    if disabledsettings == true and isdisabledsettings == false then
      log('I', '', "Disabled loading settings")
      isdisabledsettings = true
      settingsSave()
    end

    if disabledsettings == false and isdisabledsettings == true then
      isdisabledsettings = false
       log('I', '', "Enabled loading settings")
       settingsSave()
    end


  if not showUI[0] then
     return
    end

  local dataText

   --imgui app
  im.Begin(appTitle, showUI, im.WindowFlags_AlwaysAutoResize)

  if bigver == "0.28" then


    im.Text("Graphics Settings")
    im.Separator()

    if isVulkan ==  1 then
      im.TextColored(im.ImVec4(1, 0, 0, 1), "Vulkan mode!")
      im.TextColored(im.ImVec4(1, 0, 0, 1), "Game will crash when changing some settings")
      im.Separator()
    end

    if im.TreeNode1("Enable High End Graphics Features") then
    im.Separator()

    local ssaoQEnabled = im.BoolPtr(ssaoenabled)
    if im.Checkbox("Higher Quality GTAO", ssaoQEnabled) then
      ssaoenabled = ssaoQEnabled[0]
    end
    if im.IsItemHovered() then
      im.BeginTooltip()
      im.Text("Changes amount of Ambient Occlusion samples and it's radius")
      im.EndTooltip()
    end
    im.Separator()

    local pssmQEnabled = im.BoolPtr(pssmenabled)
    if im.Checkbox("Higher Shadows Resolution", pssmQEnabled) then
      pssmenabled = pssmQEnabled[0]
    end
    if im.IsItemHovered() then
      im.BeginTooltip()
      im.Text("Sets all sun shadows to 4096px")
      im.EndTooltip()
    end
    im.Separator()

    local terQEnabled = im.BoolPtr(terenabled)
    if im.Checkbox("Higher Terrain Quality", terQEnabled) then
      terenabled = terQEnabled[0]
    end
    if im.IsItemHovered() then
      im.BeginTooltip()
      im.Text("Changes distance of terrain occlusion and it's quality")
      im.EndTooltip()
    end
    im.TreePop()
    end

    im.Separator()

    if im.TreeNode1("Enable Low End Graphics Features") then
      im.Separator()

      local pssmLQEnabled = im.BoolPtr(pssmlenabled)
      if im.Checkbox("Lower Shadows Resolution", pssmLQEnabled) then
        pssmlenabled = pssmLQEnabled[0]
      end
      if im.IsItemHovered() then
        im.BeginTooltip()
        im.Text("Sets all sun shadows to 128px and disables PSSM filtering")
        im.EndTooltip()
      end
      im.Separator()

      local detailLQEnabled = im.BoolPtr(detailenabled)
      if im.Checkbox("Do not render details", detailLQEnabled) then
        detailenabled = detailLQEnabled[0]
      end
      if im.IsItemHovered() then
        im.BeginTooltip()
        im.Text("Disables decals rendering and dynamic models")
        im.EndTooltip()
      end
      im.Separator()

      local terLQEnabled = im.BoolPtr(terlenabled)
      if im.Checkbox("Lower Terrain Quality", terLQEnabled) then
        terlenabled = terLQEnabled[0]
      end
      if im.IsItemHovered() then
        im.BeginTooltip()
        im.Text("Changes distance of terrain occlusion and it's quality")
        im.EndTooltip()
      end
      im.Separator()

      local textLQEnabled = im.BoolPtr(textenabled)
      if im.Checkbox("Lower Texture Quality", textLQEnabled) then
        textenabled = textLQEnabled[0]
      end
      if im.IsItemHovered() then
        im.BeginTooltip()
        im.Text("Reduces texture resolution, lowers quality of refraction, and changes terrain texture render distance")
        im.EndTooltip()
      end
      im.TreePop()
    end

    im.Separator()

    im.Text("Groundcover Density")
    if im.IsItemHovered() then
      im.BeginTooltip()
      im.Text("Changes Groundcover density and render distance")
      im.EndTooltip()
    end
    grassBias[0] = tonumber(TorqueScriptLua.getVar("$pref::GroundCover::densityScale") or 1) * 100
    im.PushItemWidth(128)
    if im.SliderFloat("", grassBias, 0, 800, "%.1f") then
      grass = grassBias[0] / 100
      TorqueScriptLua.setVar("$pref::GroundCover::densityScale", grass)
      settingsSave()
    end
    im.SameLine()
    if im.Button("Reset Grass") then
      forcedsettings = false
      if GQ == nil then
        TorqueScriptLua.setVar("$pref::GroundCover::densityScale", 1)
      else
      TorqueScriptLua.setVar("$pref::GroundCover::densityScale", GQ)
      end
    end

    im.Separator()

    im.Text("Lod Bias")
    if im.IsItemHovered() then
      im.BeginTooltip()
      im.Text("Changes how far objects are rendering")
      im.EndTooltip()
    end
    lodBias[0] = tonumber(TorqueScriptLua.getVar("$pref::TS::detailAdjust") or 1) * 100
    im.PushItemWidth(128)
    if im.SliderFloat("##", lodBias, 15, 800, "%.1f") then
      LOD = lodBias[0] / 100
      TorqueScriptLua.setVar("$pref::TS::detailAdjust", LOD)
      settingsSave()
    end
    im.SameLine()
    if im.Button("Reset LOD") then
      forcedsettings = false
      if LODDEF == nil then
        TorqueScriptLua.setVar("$pref::TS::detailAdjust", 1)
      else
      TorqueScriptLua.setVar("$pref::TS::detailAdjust", LODDEF)
      end
    end

    im.Separator()

    if im.Button("Reset all") then
       log('I', '', "User requested reset")
       settingsSave()
      forcedsettings = false
      ssaoenabled = false
      pssmenabled = false
      terenabled = false
      pssmlenabled = false
      detailenabled = false
      terlenabled = false
      textenabled = false
      scenetree.SSAOPostFx:setSamples(16)
      if SQ == nil then
        TorqueScriptLua.setVar("$pref::Shadows::textureScalar", 1)
        TorqueScriptLua.setVar("$pref::Shadows::filterMode", "SoftShadowHighQuality")
      else
      TorqueScriptLua.setVar("$pref::Shadows::textureScalar", SQ)
      TorqueScriptLua.setVar("$pref::Shadows::filterMode", "SoftShadowHighQuality")
      end
      if GQ == nil then
        TorqueScriptLua.setVar("$pref::GroundCover::densityScale", 1)
      else
        TorqueScriptLua.setVar("$pref::GroundCover::densityScale", GQ)
      end
      if LT == nil then
        TorqueScriptLua.setVar("$pref::Terrain::lodScale", 0.75)
      else
        TorqueScriptLua.setVar("$pref::Terrain::lodScale", LT)
      end
      if LODDEF == nil then
        TorqueScriptLua.setVar("$pref::TS::detailAdjust", 1)
      else
      TorqueScriptLua.setVar("$pref::TS::detailAdjust", LODDEF)
      end
      if DC == nil then
        TorqueScriptLua.setVar("$pref::TS::maxDecalCount", 4000)
        TorqueScriptLua.setVar("$pref::TS::skipRenderDLs", 0)
      else
      TorqueScriptLua.setVar("$pref::TS::maxDecalCount", DC)
      TorqueScriptLua.setVar("$pref::TS::skipRenderDLs", 0)
      end
      if LQ == nil then
        TorqueScriptLua.setVar("$pref::Video::disablePixSpecular", 1)
        TorqueScriptLua.setVar("$pref::Video::textureReductionLevel", LQ)
        TorqueScriptLua.setVar("$pref::Reflect::refractTexScale", rTS)
        TorqueScriptLua.setVar("$pref::Terrain::detailScale", dS)
        TorqueScriptLua.setVar("$pref::Video::disablePixSpecular", 0)
        reloadTextures()
      else
        TorqueScriptLua.setVar("$pref::Video::disablePixSpecular", 1)
        TorqueScriptLua.setVar("$pref::Video::textureReductionLevel", 0)
        TorqueScriptLua.setVar("$pref::Reflect::refractTexScale", 1)
        TorqueScriptLua.setVar("$pref::Terrain::detailScale", 1)
        TorqueScriptLua.setVar("$pref::Video::disablePixSpecular", 0)
        reloadTextures()
      end
    end
    if im.IsItemHovered() then
      im.BeginTooltip()
      im.Text("Resets all settings to original")
      im.EndTooltip()
    end
    im.Separator()
    local forceSettings = im.BoolPtr(forcedsettings)
    if im.Checkbox("Force Settings (Experimental)", forceSettings) then
      forcedsettings = forceSettings[0]
    end
    if im.IsItemHovered() then
      im.BeginTooltip()
      im.Text("Always forces game to use some settings from this menu, might cause instability and performance issues")
      im.EndTooltip()
    end
    im.Separator()

    local disableSettings = im.BoolPtr(disabledsettings)
    if im.Checkbox("Disable settings persistence", disableSettings) then
      disabledsettings = disableSettings[0]
    end
    if im.IsItemHovered() then
      im.BeginTooltip()
      im.Text("Skips loading settings per user request")
      im.EndTooltip()
    end
    im.Separator()

    im.Text("Graphics API: ")
    im.SameLine()
    if isVulkan == 1 then
      im.Text("Vulkan API")
    else
      im.Text("DirectX 11")
    end
    im.Text("OS: ")
    --same but Linux
    im.SameLine()
    if isLinux == 1 then
      im.Text("Linux")
    elseif isLinux == 2 then
      im.Text("nil")
    else
      im.Text("Windows")
    end
    im.Text("FPS: ")
    im.SameLine()
    dataText = string.format("%d", getConsoleNumber("fps::instantaneous"))
    im.Text(dataText)
    im.Text("Drawcalls: ")
    im.SameLine()
    dataText =  string.format("%d", getConsoleNumber("$GFXDeviceStatistics::drawCalls"))
    im.Text(dataText)
    im.Text("Polycount: ")
    im.SameLine()
    dataText =  string.format("%d", getConsoleNumber("$GFXDeviceStatistics::polyCount"))
    im.Text(dataText)

  else
    im.TextColored(im.ImVec4(1, 0, 0, 1), "Game Version mismatch!")
    im.TextColored(im.ImVec4(1, 0, 0, 1), "Please remove mod or contact @Car_Killer")
  end

  im.End()

end

local function onClientEndMission()
  if disabledsettings == true then
    log('W', '', 'Exitting mission, resetting all changes' )
    scenetree.SSAOPostFx:setSamples(16)
    if SQ == nil then
      TorqueScriptLua.setVar("$pref::Shadows::textureScalar", 1)
      TorqueScriptLua.setVar("$pref::Shadows::filterMode", "SoftShadowHighQuality")
    else
    TorqueScriptLua.setVar("$pref::Shadows::textureScalar", SQ)
    TorqueScriptLua.setVar("$pref::Shadows::filterMode", "SoftShadowHighQuality")
    end
    if GQ == nil then
      TorqueScriptLua.setVar("$pref::GroundCover::densityScale", 1)
    else
      TorqueScriptLua.setVar("$pref::GroundCover::densityScale", GQ)
    end
    if LT == nil then
      TorqueScriptLua.setVar("$pref::Terrain::lodScale", 0.75)
    else
      TorqueScriptLua.setVar("$pref::Terrain::lodScale", LT)
    end
    if LODDEF == nil then
      TorqueScriptLua.setVar("$pref::TS::detailAdjust", 1)
    else
    TorqueScriptLua.setVar("$pref::TS::detailAdjust", LODDEF)
    end
    if DC == nil then
      TorqueScriptLua.setVar("$pref::TS::maxDecalCount", 4000)
      TorqueScriptLua.setVar("$pref::TS::skipRenderDLs", 0)
    else
    TorqueScriptLua.setVar("$pref::TS::maxDecalCount", DC)
    TorqueScriptLua.setVar("$pref::TS::skipRenderDLs", 0)
    end
    if LQ == nil then
      TorqueScriptLua.setVar("$pref::Video::disablePixSpecular", 1)
      TorqueScriptLua.setVar("$pref::Video::textureReductionLevel", LQ)
      TorqueScriptLua.setVar("$pref::Reflect::refractTexScale", rTS)
      TorqueScriptLua.setVar("$pref::Terrain::detailScale", dS)
      TorqueScriptLua.setVar("$pref::Video::disablePixSpecular", 0)
    else
      TorqueScriptLua.setVar("$pref::Video::disablePixSpecular", 1)
      TorqueScriptLua.setVar("$pref::Video::textureReductionLevel", 0)
      TorqueScriptLua.setVar("$pref::Reflect::refractTexScale", 1)
      TorqueScriptLua.setVar("$pref::Terrain::detailScale", 1)
      TorqueScriptLua.setVar("$pref::Video::disablePixSpecular", 0)
    end
  end
end

local function onExit()
   log('I', '', "Quitting game, resetting all changes")
  scenetree.SSAOPostFx:setSamples(16)
  if SQ == nil then
    TorqueScriptLua.setVar("$pref::Shadows::textureScalar", 1)
    TorqueScriptLua.setVar("$pref::Shadows::filterMode", "SoftShadowHighQuality")
  else
  TorqueScriptLua.setVar("$pref::Shadows::textureScalar", SQ)
  TorqueScriptLua.setVar("$pref::Shadows::filterMode", "SoftShadowHighQuality")
  end
  if GQ == nil then
    TorqueScriptLua.setVar("$pref::GroundCover::densityScale", 1)
  else
    TorqueScriptLua.setVar("$pref::GroundCover::densityScale", GQ)
  end
  if LT == nil then
    TorqueScriptLua.setVar("$pref::Terrain::lodScale", 0.75)
  else
    TorqueScriptLua.setVar("$pref::Terrain::lodScale", LT)
  end
  if LODDEF == nil then
    TorqueScriptLua.setVar("$pref::TS::detailAdjust", 1)
  else
  TorqueScriptLua.setVar("$pref::TS::detailAdjust", LODDEF)
  end
  if DC == nil then
    TorqueScriptLua.setVar("$pref::TS::maxDecalCount", 4000)
    TorqueScriptLua.setVar("$pref::TS::skipRenderDLs", 0)
  else
  TorqueScriptLua.setVar("$pref::TS::maxDecalCount", DC)
  TorqueScriptLua.setVar("$pref::TS::skipRenderDLs", 0)
  end
  if LQ == nil then
    TorqueScriptLua.setVar("$pref::Video::disablePixSpecular", 1)
    TorqueScriptLua.setVar("$pref::Video::textureReductionLevel", LQ)
    TorqueScriptLua.setVar("$pref::Reflect::refractTexScale", rTS)
    TorqueScriptLua.setVar("$pref::Terrain::detailScale", dS)
    TorqueScriptLua.setVar("$pref::Video::disablePixSpecular", 0)
  else
    TorqueScriptLua.setVar("$pref::Video::disablePixSpecular", 1)
    TorqueScriptLua.setVar("$pref::Video::textureReductionLevel", 0)
    TorqueScriptLua.setVar("$pref::Reflect::refractTexScale", 1)
    TorqueScriptLua.setVar("$pref::Terrain::detailScale", 1)
    TorqueScriptLua.setVar("$pref::Video::disablePixSpecular", 0)
  end
end


local function onWindowMenuItem()
  showUI[0] = true
end

local function openUI()
  showUI[0] = true
end

local function hideUI()
  showUI[0] = false
end

local function toggleUI()
  if showUI[0] then
    hideUI()
  else
    openUI()
  end
end

M.show = openUI
M.hide = hideUI
M.toggle = toggleUI
M.onExtensionLoaded = onExtensionLoaded
M.onClientEndMission = onClientEndMission
M.onUpdate = onUpdate
M.onExit = onExit
return M
--Written in Visual Studio Code with Yukino Yukinoshita theme by Car_Killer 2022